@@ -25,14 +25,13 @@ from cinderclient.v1 import client
 
 def _stub_volume(**kwargs):
     volume = {
-        'id': '1234',
+        'id': '00000000-0000-0000-0000-000000000000',
         'display_name': None,
         'display_description': None,
         "attachments": [],
         "bootable": "false",
         "availability_zone": "cinder",
         "created_at": "2012-08-27T00:00:00.000000",
-        "id": '00000000-0000-0000-0000-000000000000',
         "metadata": {},
         "size": 1,
         "snapshot_id": None,
