@@ -66,6 +66,104 @@ class ShellTest(utils.TestCase):
         return self.shell.cs.assert_called(method, url, body,
                                            partial_body, **kwargs)
 
+    @ddt.data({'resource': None, 'query_url': None},
+              {'resource': 'volume', 'query_url': '?resource=volume'},
+              {'resource': 'group', 'query_url': '?resource=group'})
+    @ddt.unpack
+    def test_list_filters(self, resource, query_url):
+        url = '/resource_filters'
+        if resource is not None:
+            url += query_url
+            self.run_command('--os-volume-api-version 3.33 '
+                             'list-filters --resource=%s' % resource)
+        else:
+            self.run_command('--os-volume-api-version 3.33 list-filters')
+
+        self.assert_called('GET', url)
+
+    @ddt.data(
+        # testcases for list volume
+        {'command':
+            'list --name=123 --filters name=456',
+         'expected':
+             '/volumes/detail?name=456'},
+        {'command':
+             'list --filters name=123',
+         'expected':
+             '/volumes/detail?name=123'},
+        {'command':
+             'list --filters metadata={key1:value1}',
+         'expected':
+             '/volumes/detail?metadata=%7B%27key1%27%3A+%27value1%27%7D'},
+        # testcases for list group
+        {'command':
+             'group-list --filters name=456',
+         'expected':
+             '/groups/detail?name=456'},
+        {'command':
+             'group-list --filters status=available',
+         'expected':
+             '/groups/detail?status=available'},
+        # testcases for list group-snapshot
+        {'command':
+             'group-snapshot-list --status=error --filters status=available',
+         'expected':
+             '/group_snapshots/detail?status=available'},
+        {'command':
+             'group-snapshot-list --filters availability_zone=123',
+         'expected':
+             '/group_snapshots/detail?availability_zone=123'},
+        # testcases for list message
+        {'command':
+             'message-list --event_id=123 --filters event_id=456',
+         'expected':
+             '/messages?event_id=456'},
+        {'command':
+             'message-list --filters request_id=123',
+         'expected':
+             '/messages?request_id=123'},
+        # testcases for list attachment
+        {'command':
+             'attachment-list --volume-id=123 --filters volume_id=456',
+         'expected':
+             '/attachments?volume_id=456'},
+        {'command':
+             'attachment-list --filters mountpoint=123',
+         'expected':
+             '/attachments?mountpoint=123'},
+        # testcases for list backup
+        {'command':
+             'backup-list --volume-id=123 --filters volume_id=456',
+         'expected':
+             '/backups/detail?volume_id=456'},
+        {'command':
+             'backup-list --filters name=123',
+         'expected':
+             '/backups/detail?name=123'},
+        # testcases for list snapshot
+        {'command':
+             'snapshot-list --volume-id=123 --filters volume_id=456',
+         'expected':
+             '/snapshots/detail?volume_id=456'},
+        {'command':
+             'snapshot-list --filters name=123',
+         'expected':
+             '/snapshots/detail?name=123'},
+        # testcases for get pools
+        {'command':
+             'get-pools --filters name=456 --detail',
+         'expected':
+             '/scheduler-stats/get_pools?detail=True&name=456'},
+        {'command':
+             'get-pools --filters name=456',
+         'expected':
+             '/scheduler-stats/get_pools?name=456'}
+    )
+    @ddt.unpack
+    def test_list_with_filters_mixed(self, command, expected):
+        self.run_command('--os-volume-api-version 3.33 %s' % command)
+        self.assert_called('GET', expected)
+
     def test_list(self):
         self.run_command('list')
         # NOTE(jdg): we default to detail currently
@@ -95,6 +193,89 @@ class ShellTest(utils.TestCase):
         self.run_command('availability-zone-list')
         self.assert_called('GET', '/os-availability-zone')
 
+    @ddt.data({'cmd': '1234 1233',
+               'body': {'instance_uuid': '1233',
+                        'connector': {},
+                        'volume_uuid': '1234'}},
+              {'cmd': '1234 1233 '
+                      '--connect True '
+                      '--ip 10.23.12.23 --host server01 '
+                      '--platform x86_xx '
+                      '--ostype 123 '
+                      '--multipath true '
+                      '--mountpoint /123 '
+                      '--initiator aabbccdd',
+               'body': {'instance_uuid': '1233',
+                        'connector': {'ip': '10.23.12.23',
+                                      'host': 'server01',
+                                      'os_type': '123',
+                                      'multipath': 'true',
+                                      'mountpoint': '/123',
+                                      'initiator': 'aabbccdd',
+                                      'platform': 'x86_xx'},
+                        'volume_uuid': '1234'}})
+    @ddt.unpack
+    def test_attachment_create(self, cmd, body):
+        command = '--os-volume-api-version 3.27 attachment-create '
+        command += cmd
+        self.run_command(command)
+        expected = {'attachment': body}
+        self.assert_called('POST', '/attachments', body=expected)
+
+    @ddt.data({'cmd': '',
+               'expected': ''},
+              {'cmd': '--volume-id 1234',
+               'expected': '?volume_id=1234'},
+              {'cmd': '--status error',
+               'expected': '?status=error'},
+              {'cmd': '--all-tenants 1',
+               'expected': '?all_tenants=1'},
+              {'cmd': '--all-tenants 1 --volume-id 12345',
+               'expected': '?all_tenants=1&volume_id=12345'},
+              {'cmd': '--all-tenants 1 --tenant 12345',
+               'expected': '?all_tenants=1&project_id=12345'},
+              {'cmd': '--tenant 12345',
+               'expected': '?all_tenants=1&project_id=12345'}
+
+              )
+    @ddt.unpack
+    def test_attachment_list(self, cmd, expected):
+        command = '--os-volume-api-version 3.27 attachment-list '
+        command += cmd
+        self.run_command(command)
+        self.assert_called('GET', '/attachments%s' % expected)
+
+    def test_attachment_show(self):
+        self.run_command('--os-volume-api-version 3.27 attachment-show 1234')
+        self.assert_called('GET', '/attachments/1234')
+
+    @ddt.data({'cmd': '1234 '
+                      '--ip 10.23.12.23 --host server01 '
+                      '--platform x86_xx '
+                      '--ostype 123 '
+                      '--multipath true '
+                      '--mountpoint /123 '
+                      '--initiator aabbccdd',
+               'body': {'connector': {'ip': '10.23.12.23',
+                                      'host': 'server01',
+                                      'os_type': '123',
+                                      'multipath': 'true',
+                                      'mountpoint': '/123',
+                                      'initiator': 'aabbccdd',
+                                      'platform': 'x86_xx'}}})
+    @ddt.unpack
+    def test_attachment_update(self, cmd, body):
+        command = '--os-volume-api-version 3.27 attachment-update '
+        command += cmd
+        self.run_command(command)
+        self.assert_called('PUT', '/attachments/1234', body={'attachment':
+                                                             body})
+
+    def test_attachment_delete(self):
+        self.run_command('--os-volume-api-version 3.27 '
+                         'attachment-delete 1234')
+        self.assert_called('DELETE', '/attachments/1234')
+
     def test_upload_to_image(self):
         expected = {'os-volume_upload_image': {'force': False,
                                                'container_format': 'bare',
@@ -371,6 +552,80 @@ class ShellTest(utils.TestCase):
         self.run_command('--os-volume-api-version 3.3 message-list')
         self.assert_called('GET', '/messages')
 
+    @ddt.data('volume', 'backup', 'snapshot', None)
+    def test_reset_state_entity_not_found(self, entity_type):
+        cmd = 'reset-state 999999'
+        if entity_type is not None:
+            cmd += ' --type %s' % entity_type
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
+
+    @ddt.data({'entity_types': [{'name': 'volume', 'version': '3.0',
+                                 'command': 'os-reset_status'},
+                                {'name': 'backup', 'version': '3.0',
+                                 'command': 'os-reset_status'},
+                                {'name': 'snapshot', 'version': '3.0',
+                                 'command': 'os-reset_status'},
+                                {'name': None, 'version': '3.0',
+                                 'command': 'os-reset_status'},
+                                {'name': 'group', 'version': '3.20',
+                                 'command': 'reset_status'},
+                                {'name': 'group-snapshot', 'version': '3.19',
+                                 'command': 'reset_status'}],
+               'r_id': ['1234'],
+               'states': ['available', 'error', None]},
+              {'entity_types': [{'name': 'volume', 'version': '3.0',
+                                 'command': 'os-reset_status'},
+                                {'name': 'backup', 'version': '3.0',
+                                 'command': 'os-reset_status'},
+                                {'name': 'snapshot', 'version': '3.0',
+                                 'command': 'os-reset_status'},
+                                {'name': None, 'version': '3.0',
+                                 'command': 'os-reset_status'},
+                                {'name': 'group', 'version': '3.20',
+                                 'command': 'reset_status'},
+                                {'name': 'group-snapshot', 'version': '3.19',
+                                 'command': 'reset_status'}],
+               'r_id': ['1234', '5678'],
+               'states': ['available', 'error', None]})
+    @ddt.unpack
+    def test_reset_state_normal(self, entity_types, r_id, states):
+        for state in states:
+            for t in entity_types:
+                if state is None:
+                    expected = {t['command']: {}}
+                    cmd = ('--os-volume-api-version '
+                           '%s reset-state %s') % (t['version'],
+                                                   ' '.join(r_id))
+                else:
+                    expected = {t['command']: {'status': state}}
+                    cmd = ('--os-volume-api-version '
+                           '%s reset-state '
+                           '--state %s %s') % (t['version'],
+                                               state, ' '.join(r_id))
+                if t['name'] is not None:
+                    cmd += ' --type %s' % t['name']
+
+                self.run_command(cmd)
+
+                name = t['name'] if t['name'] else 'volume'
+                for re in r_id:
+                    self.assert_called_anytime('POST', '/%ss/%s/action'
+                                               % (name.replace('-', '_'), re),
+                                               body=expected)
+
+    @ddt.data({'command': '--attach-status detached',
+               'expected': {'attach_status': 'detached'}},
+              {'command': '--state in-use --attach-status attached',
+               'expected': {'status': 'in-use',
+                            'attach_status': 'attached'}},
+              {'command': '--reset-migration-status',
+               'expected': {'migration_status': 'none'}})
+    @ddt.unpack
+    def test_reset_state_volume_additional_status(self, command, expected):
+        self.run_command('reset-state %s 1234' % command)
+        expected = {'os-reset_status': expected}
+        self.assert_called('POST', '/volumes/1234/action', body=expected)
+
     def test_snapshot_list_with_metadata(self):
         self.run_command('--os-volume-api-version 3.22 '
                          'snapshot-list --metadata key1=val1')
